var matchDateIso = /(\d{4})-(\d{2})-(\d{2})(?:(T)(\d{2}):(\d{2}):(\d{2}(?:\.\d{1,3})?))?(?:(Z)|(?:([+-])(\d{2}):(\d{2})))?/i;
var matchInteger = /^\d+$/i;
var matchFloat = /^\d+(?:\.\d+)$/i;


function isIsoDate(str) {
    return matchDateIso.test(str);
}

//===============================================================================
// Parse ISO 8601 Date Format date string and return a date - equivalent to https://developer.mozilla.org/en/JavaScript/Reference/Global_Objects/Date/parse
// Found here: n8v.enteuxis.org/2010/12/parsing-iso-8601-dates-in-javascript/
// some fixes added
//===============================================================================
function parseIsoDate(dateStr) {

    // parenthese matches:
    // year month day    hours minutes seconds  
    // dotmilliseconds 
    // tzstring plusminus hours minutes
    var d = dateStr.match(matchDateIso);

    //                                     0                                1       2     3     4    5     6     7         8          9    10    11  
    // "2010-12-07T01:02:03.123-01:12" => ["2010-12-07T01:02:03.123-01:12", "2010", "12", "07", "T", "01", "02", "03.123", undefined, "-", "01", "12"]
    // "2010-12-07T01:02:03-01:12"     => ["2010-12-07T01:02:03-01:12", "2010", "12", "07", "T", "01", "02", "03", undefined, "-", "01", "12"]
    // "2010-12-07T01:02:03Z"          => ["2010-12-07T01:02:03Z", "2010", "12", "07", "T", "01", "02", "03", "Z", undefined, undefined, undefined]
    // "2010-12-07T01:02:03"           => ["2010-12-07T01:02:03", "2010", "12", "07", "T", "01", "02", "03", undefined, undefined, undefined, undefined]
    // "2010-12-07"                    => ["2010-12-07", "2010", "12", "07", undefined, undefined, undefined, undefined, undefined, undefined, undefined, undefined]

    if (!d) {
        throw "Couldn't parse ISO 8601 date string '" + dateStr + "'";
    }

    // parse strings, leading zeros into proper types
    var i;
    for (i = 1; i < 11; i++) {
        if (matchInteger.test(d[i])) {
            d[i] = parseInt(d[i], 10);
        }
        if (matchFloat.test(d[i])) {
            d[i] = parseFloat(d[i]);
        }
    }

    // fix undefined numeric values
    var fixUndef = [1, 2, 3, 5, 6, 7, 10, 11];
    for (i in fixUndef) {
        if (fixUndef.hasOwnProperty(i)) {
            if (!d[fixUndef[i]]) {
                d[fixUndef[i]] = 0;
            }
        }
    }

    var hasTimezone = d[8] === "Z" || d[9] === "-" || d[9] === "+";

    var res;
    if (hasTimezone) {
        // assume date is UTC
        res = new Date(Date.UTC(d[1], d[2] - 1, d[3], d[5], d[6], d[7]));
    } else {
        // assume date is localtime
        res = new Date(d[1], d[2] - 1, d[3], d[5], d[6], d[7]);
    }

    // if there's a timezone, calculate it
    if (hasTimezone && d[10]) {
        var offset = d[10] * 60 * 60 * 1000;
        if (d[11]) {
            offset += d[11] * 60 * 1000;
        }
        if (d[9] === "-") {
            offset *= -1;
        }

        res = new Date(res.getTime() - offset);
    }

    return res;
};

function formatIsoDate(date) {
    return date.toISOString();
}

module.exports = {
    testStr: isIsoDate,
    testObject: function (date) {
        return date instanceof Date;
    },
    parse: parseIsoDate,
    format: formatIsoDate
}