var expect = require("chai").expect;
var qsjs = require("../src/app.js");

function test(str, json, target) {
    var target = target || qsjs();
    it("json->string", function () {
        expect(str).to.equal(target.encode(json));
    });

    it("string->json", function () {
        expect(json).to.deep.equal(target.decode(str));
    });
}

describe("JSON<->QueryString", function () {
    describe("single property object", function () {
        var expected = "value=data";
        var actual = { value: "data" }
        test(expected, actual);
    });

    describe("single property object with whitespace", function () {
        var expected = "value=first+second";
        var actual = { value: "first second" }
        test(expected, actual);
    });

    describe("multiple property object", function () {
        var actual = { value1: "data1", value2: "data2" };
        var expected = "value1=data1&value2=data2";
        test(expected, actual);
    });

    describe("deep object", function () {
        var actual = { deep: { value: "data" } };
        var expected = "deep.value=data";
        test(expected, actual);
    });

    describe("object with array", function () {
        var actual = { array: ["a", "b"] };
        var expected = "array(0)=a&array(1)=b";
        test(expected, actual);
    });

    describe("object with integer value", function () {
        var actual = { value: 1 };
        var expected = "value=1";
        test(expected, actual);
    });

    describe("object with iso date value", function () {
        var actual = { value: new Date(1445301615000) };
        var expected = "value=2015-10-20T00%3A40%3A15.000Z";
        test(expected, actual);
    });

    describe("object with dotNet date value", function () {
        var actual = { value: new Date(1445301615000) };
        var expected = "value=%2FDate(1445301615000)%2F";
        test(expected, actual, qsjs({ useDotNetDateFormat: true }));
    });

    describe("object with float value", function () {
        var actual = { value: 1.2 };
        var expected = "value=1.2";
        test(expected, actual);
    });

    describe("array of objects", function () {
        var actual = [{ valueOfA: "a" }, { valueOfB: "b" }];
        var expected = "(0).valueOfA=a&(1).valueOfB=b";
        test(expected, actual);
    });

    describe("array of strings", function () {
        var actual = ["a", "b", "c"];
        var expected = "(0)=a&(1)=b&(2)=c";
        test(expected, actual);
    });

    describe("array of integers", function () {
        var actual = [1, 2, 3];
        var expected = "(0)=1&(1)=2&(2)=3";
        test(expected, actual);
    });

    describe("array of float", function () {
        var actual = [1.2, 2.3, 3.4];
        var expected = "(0)=1.2&(1)=2.3&(2)=3.4";
        test(expected, actual);
    });
});