var apply = require("../src/_apply.js");
var expect = require("chai").expect;
var assert = require("chai").assert;

describe("apply", function () {
    describe("throws on wrong path", function () {
        it("to property in array", function () {
            assert.throws(() => apply([], ["prop"], null), /.*/);
        });

        it("to property in deep array", function () {
            assert.throws(() => apply({ prop: [] }, ["prop", "inner"], null), /.*/)
        });

        it("to index on object", function () {
            assert.throws(() => apply({}, [0], null), /.*/)
        });

        it("to deep array", function () {
            assert.throws(() => apply({ prop: {} }, ["prop", 0], null), /.*/)
        });
    })

    describe("value for", function () {
        it("plain object property", function () {
            var target = { prop: "before" };
            apply(target, ["prop"], "after");
            expect(target.prop).to.equal("after");
        })

        it("plain object deep property", function () {
            var target = { prop: { inner: "before" } };
            apply(target, ["prop", "inner"], "after");
            expect(target.prop.inner).to.equal("after");
        })

        it("array item value", function () {
            var target = ["before"];
            apply(target, [0], "after");
            expect(target[0]).to.equal("after");
        })

        it("array non zero-index item value", function () {
            var target = [null, "before"];
            apply(target, [1], "after");
            expect(target[1]).to.equal("after");
        })

        it("deep array item value", function () {
            var target = [["before"]];
            apply(target, [0, 0], "after");
            expect(target[0][0]).to.equal("after");
        })

        it("deep array non zero-index item value", function () {
            var target = [null, [null, "before"]];
            apply(target, [1, 1], "after");
            expect(target[1][1]).to.equal("after");
        })

        it("deep object in array property", function () {
            var target = [null, { prop: "before" }];
            apply(target, [1, "prop"], "after");
            expect(target[1].prop).to.equal("after");
        })

        it("deep array item in object property", function () {
            var target = { prop: [null, "before"] };
            apply(target, ["prop", 1], "after");
            expect(target.prop[1]).to.equal("after");
        })
    });

    describe("create", function () {
        it("plain object property", function () {
            var target = {};
            apply(target, ["other"], "value");
            expect(target.other).to.equal("value");
        })

        it("array new-index item", function () {
            var target = [null];
            apply(target, [2], "value");
            expect(target[2]).to.equal("value");
            expect(target.length).to.equal(3);
        })

        it("object deep property", function () {
            var target = {};
            apply(target, ["prop", "deep"], "value");
            expect(target.prop.deep).to.equal("value");
        })

        it("array item deep in object", function () {
            var target = {};
            apply(target, ["prop", 0], "value");
            expect(target.prop[0]).to.equal("value");
        })
    })
});