var iterator = require("../src/_iterateObject.js");
var expect = require("chai").expect;


describe("iterate", function () {
    it("plain object", function () {
        var target = { value: "data" };
        var actual = [];
        iterator(target, function (node, start) {
            actual.push({ item: node, start: start });
        })

        var root = { id: null, type: "object", value: target };
        var item = { id: "value", type: "simple", value: "data", parent: root };
        var expected = [{ item: root, start: true }, { item: item, start: true }, { item: item, start: false }, { item: root, start: false }]
        expect(expected).to.deep.equal(actual);
    });

    it("array", function () {
        var target = ["data"];
        var actual = [];
        iterator(target, function (node, start) {
            actual.push({ item: node, start: start });
        })

        var root = { id: null, type: "array", value: target };
        var item = { id: 0, type: "simple", value: "data", parent: root };
        var expected = [{ item: root, start: true }, { item: item, start: true }, { item: item, start: false }, { item: root, start: false }]
        expect(expected).to.deep.equal(actual);
    });

    it("deep array in object", function () {
        var target = { value: ["data"] };
        var actual = [];
        iterator(target, function (node, start) {
            actual.push({ item: node, start: start });
        })

        var root = { id: null, type: "object", value: target };
        var item = { id: "value", type: "array", value: ["data"], parent: root };
        var arrayItem = { id: 0, type: "simple", value: "data", parent: item };
        var expected = [{ item: root, start: true }, { item: item, start: true }, { item: arrayItem, start: true }, { item: arrayItem, start: false }, { item: item, start: false }, { item: root, start: false }]
        expect(expected).to.deep.equal(actual);
    });

    it("deep object in object", function () {
        var target = { value: { inner: "data" } };
        var actual = [];
        iterator(target, function (node, start) {
            actual.push({ item: node, start: start });
        })

        var root = { id: null, type: "object", value: target };
        var item = { id: "value", type: "object", value: { inner: "data" }, parent: root };
        var inner = { id: "inner", type: "simple", value: "data", parent: item };
        var expected = [{ item: root, start: true }, { item: item, start: true }, { item: inner, start: true }, { item: inner, start: false }, { item: item, start: false }, { item: root, start: false }]
        expect(expected).to.deep.equal(actual);
    });


    it("deep object in array", function () {
        var target = [{ inner: "data" }];
        var actual = [];
        iterator(target, function (node, start) {
            actual.push({ item: node, start: start });
        })


        var root = { id: null, type: "array", value: target };
        var item = { id: 0, type: "object", value: { inner: "data" }, parent: root };
        var inner = { id: "inner", type: "simple", value: "data", parent: item };
        var expected = [{ item: root, start: true }, { item: item, start: true }, { item: inner, start: true }, { item: inner, start: false }, { item: item, start: false }, { item: root, start: false }]
        expect(expected).to.deep.equal(actual);
    });
})